@@ -1,26 +1,26 @@
 import copy
 import datetime
-from typing import Tuple, Literal, Dict, Union
-from dataclasses import dataclass
+from typing import Dict, Tuple, Union, Literal
 from functools import cached_property
+from dataclasses import dataclass
 
 import numpy as np  # type: ignore
 from astropy.wcs import WCS  # type: ignore
 from dkist_fits_specifications import spec122
 from dkist_fits_specifications.spec214 import level0 as spec214_level0
 
-from dkist_data_simulator.dataset import Dataset, key_function, Copy
+from dkist_data_simulator.dataset import Copy, Dataset, key_function
 from dkist_data_simulator.schemas import Schema, TimeKey
 
-__all__ = ['Spec122Dataset', 'Spec122Schema']
+__all__ = ["Spec122Dataset", "Spec122Schema"]
 
 
 KNOWN_INSTRUMENT_TABLES = {
-    'vbi': 'VBI',
-    'visp': 'VISP',
-    'dlnirsp': 'DL-NIRSP',
-    'cryonirsp': 'CRYO-NIRSP',
-    'vtf': 'VTF',
+    "vbi": "VBI",
+    "visp": "VISP",
+    "dlnirsp": "DL-NIRSP",
+    "cryonirsp": "CRYO-NIRSP",
+    "vtf": "VTF",
 }
 
 
@@ -29,6 +29,7 @@ class Level0Schema(Schema):
     """
     A representation of a Level 0 schema.
     """
+
     def __init__(self, instrument=None):
         if instrument:
             if instrument not in KNOWN_INSTRUMENT_TABLES.keys():
@@ -86,36 +87,43 @@ class Spec122Dataset(Dataset):
         at the data center.
     """
 
-    def __init__(self,
-                 dataset_shape: Tuple[int, ...],
-                 array_shape: Tuple[int, ...],
-                 time_delta: float,
-                 start_time: datetime.datetime = None,
-                 instrument: Literal[tuple(KNOWN_INSTRUMENT_TABLES.keys())] = None,
-                 file_schema: Union[Schema, Literal["spec122", "level0_spec214"]] = "spec122"):
+    def __init__(
+        self,
+        dataset_shape: Tuple[int, ...],
+        array_shape: Tuple[int, ...],
+        time_delta: float,
+        start_time: datetime.datetime = None,
+        instrument: Literal[tuple(KNOWN_INSTRUMENT_TABLES.keys())] = None,
+        file_schema: Union[Schema, Literal["spec122", "level0_spec214"]] = "spec122",
+    ):
 
         self.time_delta = time_delta
         self.start_time = start_time or datetime.datetime.fromisoformat(
-            TimeKey("", False, False).generate_value())
+            TimeKey("", False, False).generate_value()
+        )
 
         if file_schema == "spec122":
             file_schema = Spec122Schema(instrument)
         if file_schema == "level0_spec214":
             file_schema = Level0Spec214Schema(instrument)
 
-        super().__init__(file_schema=file_schema,
-                         dataset_shape=dataset_shape,
-                         array_shape=array_shape)
+        super().__init__(
+            file_schema=file_schema,
+            dataset_shape=dataset_shape,
+            array_shape=array_shape,
+        )
 
-        end_time = self.start_time + datetime.timedelta(seconds=self.n_files * self.time_delta)
+        end_time = self.start_time + datetime.timedelta(
+            seconds=self.n_files * self.time_delta
+        )
 
         # FITS
         self.add_constant_key("NAXIS", self.array_ndim)
         self.add_constant_key("NAXIS1", self.array_shape[2])
         self.add_constant_key("NAXIS2", self.array_shape[1])
         self.add_constant_key("NAXIS3", self.array_shape[0])
-        self.add_constant_key("DKIST011", self.start_time.isoformat('T'))
-        self.add_constant_key("DKIST012", end_time.isoformat('T'))
+        self.add_constant_key("DKIST011", self.start_time.isoformat("T"))
+        self.add_constant_key("DKIST012", end_time.isoformat("T"))
         self.add_constant_key("ORIGIN", "National Solar Observatory")
         self.add_constant_key("TELESCOP", "Daniel K. Inouye Solar Telescope")
         self.add_constant_key("OBSERVAT", "Haleakala High Altitude Observatory Site")
@@ -132,9 +140,10 @@ class Spec122Dataset(Dataset):
             # Add a Copy action class to the header for each target key we
             # want, but set the flag so the target key is only added if the
             # source key is in the generated header.
-            self.add_generator_function(tuple(level0_214_122_map.keys()),
-                                        lambda self, key: Copy(level0_214_122_map[key],
-                                                               if_present=True))
+            self.add_generator_function(
+                tuple(level0_214_122_map.keys()),
+                lambda self, key: Copy(level0_214_122_map[key], if_present=True),
+            )
 
     @property
     def data(self):
@@ -145,11 +154,13 @@ class Spec122Dataset(Dataset):
     ###########################################################################
     @key_function("DATE")
     def date(self, key: str):
-        return datetime.datetime.now().isoformat('T')
+        return datetime.datetime.now().isoformat("T")
 
     @key_function("DATE-OBS")
     def date_obs(self, key: str):
-        return (self.start_time + datetime.timedelta(seconds=self.index * self.time_delta)).isoformat('T')
+        return (
+            self.start_time + datetime.timedelta(seconds=self.index * self.time_delta)
+        ).isoformat("T")
 
     ###########################################################################
     # WCS
