@@ -1,26 +1,25 @@
 import copy
 import datetime
 from abc import abstractmethod
-from typing import Tuple, Literal
+from typing import List, Tuple, Literal, Optional
 from dataclasses import dataclass
-from functools import cached_property
-from astropy.coordinates import EarthLocation, SkyCoord, GCRS
+
+import astropy.units as u
+import numpy as np
+from astropy.coordinates import GCRS, EarthLocation, SkyCoord
 from astropy.coordinates.matrix_utilities import rotation_matrix
 from astropy.time import Time
+from dkist_fits_specifications import spec214
+from hashids import Hashids
+from scipy.stats import stats
 from sunpy.coordinates import HeliographicStonyhurst, sun
 from sunpy.util.decorators import cached_property_based_on
 
-from hashids import Hashids
-import numpy as np
-from scipy.stats import stats
-import astropy.units as u
-from dkist_fits_specifications import spec214
-
-from dkist_data_simulator.dataset import Dataset, key_function, Omit
+from dkist_data_simulator.dataset import Dataset, Omit, key_function
 from dkist_data_simulator.schemas import Schema, TimeKey
 from dkist_data_simulator.spec122 import KNOWN_INSTRUMENT_TABLES
 
-__all__ = ['Spec214Dataset', 'Spec214Schema']
+__all__ = ["Spec214Dataset", "Spec214Schema"]
 
 
 @dataclass(init=False)
@@ -29,10 +28,19 @@ class Spec214Schema(Schema):
     A representation of the 214 schema.
     """
 
-    def __init__(self, naxis: int, dnaxis: int, deaxes: int, daaxes: int, instrument=None, random=None):
+    def __init__(
+        self,
+        naxis: int,
+        dnaxis: int,
+        deaxes: int,
+        daaxes: int,
+        instrument=None,
+        random=None,
+    ):
         random = random or np.random.default_rng()
-        sections = spec214.load_expanded_spec214(NAXIS=naxis, DNAXIS=dnaxis,
-                                                 DEAXES=deaxes, DAAXES=daaxes)
+        sections = spec214.load_expanded_spec214(
+            NAXIS=naxis, DNAXIS=dnaxis, DEAXES=deaxes, DAAXES=daaxes
+        )
         sections.pop("compression")
         if instrument:
             if instrument not in KNOWN_INSTRUMENT_TABLES.keys():
@@ -52,6 +60,7 @@ class Spec214Dataset(Dataset):
     """
     Generate a collection of FITS files which form a single dataset
     """
+
     _subclasses = {}
 
     def __init_subclass__(cls, **kwargs):
@@ -59,32 +68,39 @@ class Spec214Dataset(Dataset):
         if hasattr(cls, "name") and cls is not Spec214Dataset:
             Spec214Dataset._subclasses[cls.name] = cls
 
-    def __init__(self,
-                 dataset_shape: Tuple[int, ...],
-                 array_shape: Tuple[int, ...],
-                 *,
-                 time_delta: float,
-                 start_time: datetime.datetime = None,
-                 instrument: Literal[tuple(KNOWN_INSTRUMENT_TABLES.keys())] = None,
-                 swnbin: int = 0,
-                 hwnbin: int = 0,
-                 swnroi: int = 0,
-                 hwnroi: int = 0):
+    def __init__(
+        self,
+        dataset_shape: Tuple[int, ...],
+        array_shape: Tuple[int, ...],
+        *,
+        time_delta: float,
+        start_time: datetime.datetime = None,
+        instrument: Literal[tuple(KNOWN_INSTRUMENT_TABLES.keys())] = None,
+        swnbin: int = 0,
+        hwnbin: int = 0,
+        swnroi: int = 0,
+        hwnroi: int = 0,
+    ):
 
         # We have to recalculate a bunch of stuff from the super init here to
         # expand the schema before we call the super constructor.
         data_shape = tuple(d for d in array_shape if d != 1)
-        super().__init__(file_schema=Spec214Schema(naxis=len(array_shape),
-                                                   dnaxis=len(dataset_shape),
-                                                   deaxes=len(dataset_shape) - len(data_shape),
-                                                   daaxes=len(data_shape),
-                                                   instrument=instrument),
-                         dataset_shape=dataset_shape,
-                         array_shape=array_shape)
+        super().__init__(
+            file_schema=Spec214Schema(
+                naxis=len(array_shape),
+                dnaxis=len(dataset_shape),
+                deaxes=len(dataset_shape) - len(data_shape),
+                daaxes=len(data_shape),
+                instrument=instrument,
+            ),
+            dataset_shape=dataset_shape,
+            array_shape=array_shape,
+        )
 
         self.time_delta = time_delta
         self.start_time = start_time or datetime.datetime.fromisoformat(
-            TimeKey("", False, False).generate_value())
+            TimeKey("", False, False).generate_value()
+        )
 
         if instrument:
             self.add_constant_key("INSTRUME", KNOWN_INSTRUMENT_TABLES[instrument])
@@ -114,18 +130,18 @@ class Spec214Dataset(Dataset):
             self.add_constant_key(f"NBIN{n}", int(np.sqrt(nbin)))
 
         if hwnbin:
-          self.add_constant_key("HWBIN1", int(self.array_shape[1] / np.sqrt(hwnbin)))
-          self.add_constant_key("HWBIN2", int(self.array_shape[0] / np.sqrt(hwnbin)))
+            self.add_constant_key("HWBIN1", int(self.array_shape[1] / np.sqrt(hwnbin)))
+            self.add_constant_key("HWBIN2", int(self.array_shape[0] / np.sqrt(hwnbin)))
         else:
-          self.add_constant_key("HWBIN1", 1)
-          self.add_constant_key("HWBIN2", 1)
+            self.add_constant_key("HWBIN1", 1)
+            self.add_constant_key("HWBIN2", 1)
 
         if swnbin:
-          self.add_constant_key("SWBIN1", int(self.array_shape[1] / np.sqrt(swnbin)))
-          self.add_constant_key("SWBIN2", int(self.array_shape[0] / np.sqrt(swnbin)))
+            self.add_constant_key("SWBIN1", int(self.array_shape[1] / np.sqrt(swnbin)))
+            self.add_constant_key("SWBIN2", int(self.array_shape[0] / np.sqrt(swnbin)))
         else:
-          self.add_constant_key("SWBIN1", 1)
-          self.add_constant_key("SWBIN2", 1)
+            self.add_constant_key("SWBIN1", 1)
+            self.add_constant_key("SWBIN2", 1)
 
         self.add_constant_key("SWNROI", swnroi)
         self.add_constant_key("HWNROI", hwnroi)
@@ -133,7 +149,7 @@ class Spec214Dataset(Dataset):
         # Solarnet
         self.add_constant_key("BTYPE", "phot.count")
         self.add_constant_key("SOLARNET", 1.0)
-        self.add_constant_key("EXTNAME", 'OBSERVATION')
+        self.add_constant_key("EXTNAME", "OBSERVATION")
         self.add_constant_key("INFO_URL", "https://youtu.be/YddwkMJG1Jo")
 
         # WCS
@@ -165,7 +181,7 @@ class Spec214Dataset(Dataset):
             self.add_generator_function(f"DINDEX{k}", type(self).dindex)
 
         # dtypes need to be generated as a coherent set, so it needs state
-        self._dtypes = None
+        self._dtypes: Optional[List[str]] = None
 
         # d index keys
         for d in range(1, self.dataset_ndim + 1):
@@ -184,7 +200,7 @@ class Spec214Dataset(Dataset):
     ###########################################################################
     @key_function("DATE")
     def date(self, key: str):
-        return datetime.datetime.now().isoformat('T')
+        return datetime.datetime.now().isoformat("T")
 
     @key_function(
         "DATE-BEG",
@@ -210,14 +226,16 @@ class Spec214Dataset(Dataset):
         if key == "TELAPSE":
             return (frame_end - frame_start).seconds
 
-        return ret.isoformat('T')
+        return ret.isoformat("T")
 
     @property
     def observer(self):
         date_avg = Time(self.date_obs("DATE-AVG"))
-        dkist_pos = EarthLocation.of_site('dkist')
+        dkist_pos = EarthLocation.of_site("dkist")
         dkist_itrs = dkist_pos.get_itrs(date_avg)
-        dkist_hgs = dkist_pos.get_gcrs(date_avg).transform_to(HeliographicStonyhurst(obstime=date_avg))
+        dkist_hgs = dkist_pos.get_gcrs(date_avg).transform_to(
+            HeliographicStonyhurst(obstime=date_avg)
+        )
         return dkist_itrs, dkist_hgs
 
     @key_function(
@@ -234,7 +252,7 @@ class Spec214Dataset(Dataset):
             return getattr(dkist_itrs, attribute).to_value(u.m)
 
         if key == "OBS_VR":
-            return dkist_hgs.velocity.d_z.to_value(u.km/u.s)
+            return dkist_hgs.velocity.d_z.to_value(u.km / u.s)
 
     @key_function("NAXISn")
     def naxis(self, key: str):
@@ -246,6 +264,21 @@ class Spec214Dataset(Dataset):
     def nbin(self, key: str):
         return 1
 
+    @key_function(
+        "MAXIS",
+        "MAXIS1",
+        "MAXIS2",
+        "MINDEX1",
+        "MINDEX2",
+        "VBISTP",
+    )
+    def moasic_keys(self, key: str):
+        """
+        These are generated in a method so they are skipped by default but then
+        can be overridden.
+        """
+        return Omit()
+
     ###########################################################################
     # WCS
     ###########################################################################
@@ -293,7 +326,6 @@ class Spec214Dataset(Dataset):
         if "ORI" in key:
             return naxisn / self.roin * (int(key[3]) - 1)
 
-
     ###########################################################################
     # Dataset
     ###########################################################################
@@ -320,23 +352,29 @@ class Spec214Dataset(Dataset):
         Generate a random set of types and then sanitise the number and
         position of any spatial axes.
         """
-        wapt_translation = {
-            "pos": "SPATIAL",
-            "em": "SPECTRAL",
-            "time": "TEMPORAL"
-        }
+        wapt_translation = {"pos": "SPATIAL", "em": "SPECTRAL", "time": "TEMPORAL"}
 
         if self._dtypes is None:
             self._dtypes = []
             for wapt in self.fits_wcs.world_axis_physical_types:
-                atype = tuple(filter(lambda x: x[0] in wapt, wapt_translation.items()))[0]
+                atype = tuple(filter(lambda x: x[0] in wapt, wapt_translation.items()))[
+                    0
+                ]
                 self._dtypes.append(atype[1])
 
             if not self._dtypes.count("SPATIAL") == 2:
-                raise ValueError("It is expected the FITS WCS describe both spatial dimensions")
+                raise ValueError(
+                    "It is expected the FITS WCS describe both spatial dimensions"
+                )
 
             for _ in range(len(self._dtypes), self.dataset_ndim):
-                self._dtypes.append(self._not_spatial_dtype())
+                # Non spatial dtypes shouldn't be duplicated
+                # TODO: Make this not terrible
+                while True:
+                    new_dtype = self._not_spatial_dtype()
+                    if new_dtype not in self._dtypes:
+                        self._dtypes.append(new_dtype)
+                        break
 
             if "TEMPORAL" not in self._dtypes and "SPECTRAL" in self._dtypes:
                 self._dtypes[self._dtypes.index("SPECTRAL")] = "TEMPORAL"
@@ -344,7 +382,6 @@ class Spec214Dataset(Dataset):
             if "TEMPORAL" not in self._dtypes:
                 self._dtypes[self._dtypes.index("STOKES")] = "TEMPORAL"
 
-
         return self._dtypes[int(key[-1]) - 1]
 
     def dunit(self, key: str):
@@ -353,12 +390,12 @@ class Spec214Dataset(Dataset):
             "SPATIAL": u.arcsec,
             "SPECTRAL": u.nm,
             "TEMPORAL": u.s,
-            "STOKES": u.one
+            "STOKES": u.one,
         }
         dtype = self.dtype(f"DTYPE{d}")
         unit = type_unit_map[dtype]
         if unit is not None:
-            return unit.to_string(format='fits')
+            return unit.to_string(format="fits")
 
     ###########################################################################
     # Stats
@@ -413,13 +450,11 @@ class AOMixin:
     AOLOOPI: Same as above but use 10 for a typical value.
     FRMRATE: This should always be 1970 Hz except in rare cases for engineering purposes
     FRMLOCK: This is similar to WFC__002 above but for the tip-tilt system. Use values as described above. They don't have to be the same as the WFC_002 values.
-    TTLOOPP, TTLOOPI: Similar to 004 and 005 above but use 0.2 and 1 for typical values, respectively.
-"""
+    TTLOOPP, TTLOOPI: Similar to 004 and 005 above but use 0.2 and 1 for typical values, respectively."""
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
-
         # Assume AO is on and Locked
         self.add_constant_key("AO_LOCK", True)
         self.add_constant_key("WFCLOCK", True)
@@ -491,6 +526,7 @@ class WCSAMixin:
 
     This takes a noticeable amount of time to compute per frame so it is opt-in.
     """
+
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
@@ -499,22 +535,26 @@ class WCSAMixin:
         self.add_constant_key("CRDATEnA")
 
     @property
-    @cached_property_based_on('index')
+    @cached_property_based_on("index")
     def fits_wcs_a(self):
         """
         This property takes in the `fits_wcs` property and attempts to build a
         mostly accurate Equatorial WCS from the HPC one.
         """
         new_wcs = copy.deepcopy(self.fits_wcs)
-        lat_ind = [i for i, ctype in enumerate(new_wcs.wcs.ctype) if ctype.startswith("HPLT")][0]
-        lon_ind = [i for i, ctype in enumerate(new_wcs.wcs.ctype) if ctype.startswith("HPLN")][0]
+        lat_ind = [
+            i for i, ctype in enumerate(new_wcs.wcs.ctype) if ctype.startswith("HPLT")
+        ][0]
+        lon_ind = [
+            i for i, ctype in enumerate(new_wcs.wcs.ctype) if ctype.startswith("HPLN")
+        ][0]
 
         ctype = list(new_wcs.wcs.ctype)
         ctype[lon_ind] = "DEC--TAN"
         ctype[lat_ind] = "RA---TAN"
 
         crpix = [new_wcs.wcs.crpix[i] for i in sorted([lon_ind, lat_ind])]
-        hpc_ref = SkyCoord.from_pixel(*crpix, wcs=new_wcs, mode='wcs', origin=1)
+        hpc_ref = SkyCoord.from_pixel(*crpix, wcs=new_wcs, mode="wcs", origin=1)
         hpc_ref = hpc_ref.replicate(observer=self.observer[1])
         gcrs_frame = GCRS(obsgeoloc=self.observer[0].cartesian, obstime=hpc_ref.obstime)
 
@@ -571,8 +611,7 @@ class WCSAMixin:
         default = self.fits_wcs_a.wcs.pc[j, i]
         return self.fits_wcs_a.to_header().get(key, default)
 
-    @key_function("SPECSYSA",
-                  "VELOSYSA")
+    @key_function("SPECSYSA", "VELOSYSA")
     def specsys(self, key: str):
         if key.startswith("SPECSYS"):
             return "TOPOCENT"
