@@ -1,6 +1,5 @@
-import numpy as np
-
 import astropy.units as u
+import numpy as np
 from astropy.wcs import WCS
 
 from .core import Spec214Dataset
@@ -11,17 +10,21 @@ class BaseVISPDataset(Spec214Dataset):
     A base class for VISP datasets.
     """
 
-    def __init__(self,
-                 n_maps,
-                 n_steps,
-                 n_stokes,
-                 time_delta,
-                 *,
-                 linewave,
-                 detector_shape=(4096, 4096)):
+    def __init__(
+        self,
+        n_maps,
+        n_steps,
+        n_stokes,
+        time_delta,
+        *,
+        linewave,
+        detector_shape=(4096, 4096)
+    ):
 
         if (not n_maps) or (not n_steps):
-            raise NotImplementedError("Support for less than 4D VISP datasets is not implemented.")
+            raise NotImplementedError(
+                "Support for less than 4D VISP datasets is not implemented."
+            )
 
         array_shape = [1] + list(detector_shape)
 
@@ -29,7 +32,12 @@ class BaseVISPDataset(Spec214Dataset):
         if n_stokes:
             dataset_shape_rev += [n_stokes]
 
-        super().__init__(dataset_shape_rev[::-1], array_shape, time_delta=time_delta, instrument='visp')
+        super().__init__(
+            dataset_shape_rev[::-1],
+            array_shape,
+            time_delta=time_delta,
+            instrument="visp",
+        )
 
         self.add_constant_key("DTYPE1", "SPECTRAL")
         self.add_constant_key("DTYPE2", "SPATIAL")
@@ -76,15 +84,23 @@ class SimpleVISPDataset(BaseVISPDataset):
     @property
     def fits_wcs(self):
         if self.array_ndim != 3:
-            raise ValueError("VISP dataset generator expects a three dimensional FITS WCS.")
+            raise ValueError(
+                "VISP dataset generator expects a three dimensional FITS WCS."
+            )
 
         w = WCS(naxis=self.array_ndim)
-        w.wcs.crpix = self.array_shape[1] / 2, self.array_shape[0] / 2, self.file_index[-1] * -1
+        w.wcs.crpix = (
+            self.array_shape[1] / 2,
+            self.array_shape[0] / 2,
+            self.file_index[-1] * -1,
+        )
         # TODO: linewav is not a good centre point
         w.wcs.crval = self.linewave.to_value(u.nm), 0, 0
-        w.wcs.cdelt = (self.spectral_scale.to_value(u.nm / u.pix),
-                       self.plate_scale.to_value(u.arcsec / u.pix),
-                       self.slit_width.to_value(u.arcsec))
+        w.wcs.cdelt = (
+            self.spectral_scale.to_value(u.nm / u.pix),
+            self.plate_scale.to_value(u.arcsec / u.pix),
+            self.slit_width.to_value(u.arcsec),
+        )
         w.wcs.cunit = "nm", "arcsec", "arcsec"
         w.wcs.ctype = "WAVE", "HPLT-TAN", "HPLN-TAN"
         w.wcs.pc = np.identity(self.array_ndim)
