@@ -1,6 +1,5 @@
-import numpy as np
-
 import astropy.units as u
+import numpy as np
 from astropy.wcs import WCS
 
 from .core import Spec214Dataset
@@ -11,17 +10,21 @@ class BaseVTFDataset(Spec214Dataset):
     A base class for VTF datasets.
     """
 
-    def __init__(self,
-                 n_wave,
-                 n_repeats,
-                 n_stokes,
-                 time_delta,
-                 *,
-                 linewave,
-                 detector_shape=(4096, 4096)):
+    def __init__(
+        self,
+        n_wave,
+        n_repeats,
+        n_stokes,
+        time_delta,
+        *,
+        linewave,
+        detector_shape=(4096, 4096)
+    ):
 
         if not n_wave or not n_repeats:
-            raise NotImplementedError("Support for less than 4D VTF datasets is not implemented.")
+            raise NotImplementedError(
+                "Support for less than 4D VTF datasets is not implemented."
+            )
 
         array_shape = list(detector_shape)
 
@@ -29,7 +32,12 @@ class BaseVTFDataset(Spec214Dataset):
         if n_stokes:
             dataset_shape_rev += [n_stokes]
 
-        super().__init__(dataset_shape_rev[::-1], array_shape, time_delta=time_delta, instrument='vtf')
+        super().__init__(
+            dataset_shape_rev[::-1],
+            array_shape,
+            time_delta=time_delta,
+            instrument="vtf",
+        )
 
         self.add_constant_key("DTYPE1", "SPATIAL")
         self.add_constant_key("DPNAME1", "spatial x")
